@@ -12,14 +12,8 @@ import (
 	"text/template"
 
 	"github.com/Masterminds/sprig"
-	"github.com/clarketm/json"
-	ctconfig "github.com/coreos/container-linux-config-transpiler/config"
-	cttypes "github.com/coreos/container-linux-config-transpiler/config/types"
-	igntypes "github.com/coreos/ignition/config/v2_2/types"
-	"github.com/ghodss/yaml"
 	"github.com/golang/glog"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	runtime "k8s.io/apimachinery/pkg/runtime"
+	configv1 "github.com/openshift/api/config/v1"
 
 	mcfgv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 	ctrlcommon "github.com/openshift/machine-config-operator/pkg/controller/common"
@@ -33,20 +27,9 @@ type RenderConfig struct {
 }
 
 const (
-	filesDir = "files"
-	unitsDir = "units"
-
-	// TODO: these constants are wrong, they should match what is reported by the infrastructure provider
-	platformAWS       = "aws"
-	platformAzure     = "azure"
-	platformBaremetal = "baremetal"
-	platformGCP       = "gcp"
-	platformOpenStack = "openstack"
-	platformLibvirt   = "libvirt"
-	platformNone      = "none"
-	platformVSphere   = "vsphere"
-	platformBase      = "_base"
-	platformOvirt     = "ovirt"
+	filesDir     = "files"
+	unitsDir     = "units"
+	platformBase = "_base"
 )
 
 // generateTemplateMachineConfigs returns MachineConfig objects from the templateDir and a config object
@@ -137,21 +120,34 @@ func GenerateMachineConfigsForRole(config *RenderConfig, role, templateDir strin
 	return cfgs, nil
 }
 
-func platformFromControllerConfigSpec(ic *mcfgv1.ControllerConfigSpec) (string, error) {
-	switch ic.Platform {
+func platformStringFromControllerConfigSpec(ic *mcfgv1.ControllerConfigSpec) (string, error) {
+	if ic.Infra == nil {
+		ic.Infra = &configv1.Infrastructure{
+			Status: configv1.InfrastructureStatus{},
+		}
+	}
+
+	if ic.Infra.Status.PlatformStatus == nil {
+		ic.Infra.Status.PlatformStatus = &configv1.PlatformStatus{
+			Type: "",
+		}
+	}
+
+	switch ic.Infra.Status.PlatformStatus.Type {
 	case "":
 		// if Platform is nil, return nil platform and an error message
-		return "", fmt.Errorf("cannot generate MachineConfigs when no platform is set")
+		return "", fmt.Errorf("cannot generate MachineConfigs when no platformStatus.type is set")
 	case platformBase:
 		return "", fmt.Errorf("platform _base unsupported")
-	case platformAWS, platformAzure, platformBaremetal, platformGCP, platformOpenStack, platformLibvirt, platformOvirt, platformVSphere, platformNone:
-		return ic.Platform, nil
+	case configv1.AWSPlatformType, configv1.AzurePlatformType, configv1.BareMetalPlatformType, configv1.GCPPlatformType, configv1.OpenStackPlatformType, configv1.LibvirtPlatformType, configv1.OvirtPlatformType, configv1.VSpherePlatformType, configv1.NonePlatformType:
+		return strings.ToLower(string(ic.Infra.Status.PlatformStatus.Type)), nil
 	default:
 		// platformNone is used for a non-empty, but currently unsupported platform.
 		// This allows us to incrementally roll out new platforms across the project
 		// by provisioning platforms before all support is added.
-		glog.Warningf("Warning: the controller config referenced an unsupported platform: %s", ic.Platform)
-		return platformNone, nil
+		glog.Warningf("Warning: the controller config referenced an unsupported platform: %v", ic.Infra.Status.PlatformStatus.Type)
+
+		return strings.ToLower(string(configv1.NonePlatformType)), nil
 	}
 }
 
@@ -188,7 +184,7 @@ func filterTemplates(toFilter map[string]string, path string, config *RenderConf
 }
 
 func generateMachineConfigForName(config *RenderConfig, role, name, templateDir, path string, commonAdded *bool) (*mcfgv1.MachineConfig, error) {
-	platform, err := platformFromControllerConfigSpec(config.ControllerConfigSpec)
+	platformString, err := platformStringFromControllerConfigSpec(config.ControllerConfigSpec)
 	if err != nil {
 		return nil, err
 	}
@@ -196,7 +192,7 @@ func generateMachineConfigForName(config *RenderConfig, role, name, templateDir,
 	platformDirs := []string{}
 	if !*commonAdded {
 		// Loop over templates/common which applies everywhere
-		for _, dir := range []string{platformBase, platform} {
+		for _, dir := range []string{platformBase, platformString} {
 			basePath := filepath.Join(templateDir, "common", dir)
 			exists, err := existsDir(basePath)
 			if err != nil {
@@ -210,7 +206,7 @@ func generateMachineConfigForName(config *RenderConfig, role, name, templateDir,
 		*commonAdded = true
 	}
 	// And now over the target e.g. templates/master/00-master,01-master-container-runtime,01-master-kubelet
-	for _, dir := range []string{platformBase, platform} {
+	for _, dir := range []string{platformBase, platformString} {
 		platformPath := filepath.Join(path, dir)
 		exists, err := existsDir(platformPath)
 		if err != nil {
@@ -266,11 +262,11 @@ func generateMachineConfigForName(config *RenderConfig, role, name, templateDir,
 		return vs
 	}
 
-	ignCfg, err := transpileToIgn(keySortVals(files), keySortVals(units))
+	ignCfg, err := ctrlcommon.TranspileCoreOSConfigToIgn(keySortVals(files), keySortVals(units))
 	if err != nil {
-		return nil, fmt.Errorf("error transpiling ct config to Ignition config: %v", err)
+		return nil, fmt.Errorf("error transpiling CoreOS config to Ignition config: %v", err)
 	}
-	mcfg, err := MachineConfigFromIgnConfig(role, name, ignCfg)
+	mcfg, err := ctrlcommon.MachineConfigFromIgnConfig(role, name, ignCfg)
 	if err != nil {
 		return nil, fmt.Errorf("error creating MachineConfig from Ignition config: %v", err)
 	}
@@ -280,71 +276,11 @@ func generateMachineConfigForName(config *RenderConfig, role, name, templateDir,
 	return mcfg, nil
 }
 
-// MachineConfigFromIgnConfig creates a MachineConfig with the provided Ignition config
-func MachineConfigFromIgnConfig(role, name string, ignCfg interface{}) (*mcfgv1.MachineConfig, error) {
-	rawIgnCfg, err := json.Marshal(ignCfg)
-	if err != nil {
-		return nil, fmt.Errorf("error marshalling Ignition config: %v", err)
-	}
-	labels := map[string]string{
-		mcfgv1.MachineConfigRoleLabelKey: role,
-	}
-	return &mcfgv1.MachineConfig{
-		ObjectMeta: metav1.ObjectMeta{
-			Labels: labels,
-			Name:   name,
-		},
-		Spec: mcfgv1.MachineConfigSpec{
-			OSImageURL: "",
-			Config: runtime.RawExtension{
-				Raw: rawIgnCfg,
-			},
-		},
-	}, nil
-}
-
-func transpileToIgn(files, units []string) (*igntypes.Config, error) {
-	var ctCfg cttypes.Config
-
-	// Convert data to Ignition resources
-	for _, d := range files {
-		f := new(cttypes.File)
-		if err := yaml.Unmarshal([]byte(d), f); err != nil {
-			return nil, fmt.Errorf("failed to unmarshal file into struct: %v", err)
-		}
-
-		// Add the file to the config
-		ctCfg.Storage.Files = append(ctCfg.Storage.Files, *f)
-	}
-
-	for _, d := range units {
-		u := new(cttypes.SystemdUnit)
-		if err := yaml.Unmarshal([]byte(d), u); err != nil {
-			return nil, fmt.Errorf("failed to unmarshal systemd unit into struct: %v", err)
-		}
-
-		// Add the unit to the config
-		ctCfg.Systemd.Units = append(ctCfg.Systemd.Units, *u)
-	}
-
-	ignCfg, rep := ctconfig.Convert(ctCfg, "", nil)
-	if rep.IsFatal() {
-		return nil, fmt.Errorf("failed to convert config to Ignition config %s", rep)
-	}
-
-	return &ignCfg, nil
-}
-
 // renderTemplate renders a template file with values from a RenderConfig
 // returns the rendered file data
 func renderTemplate(config RenderConfig, path string, b []byte) ([]byte, error) {
 	funcs := sprig.TxtFuncMap()
 	funcs["skip"] = skipMissing
-	funcs["etcdServerCertDNSNames"] = etcdServerCertDNSNames
-	funcs["etcdPeerCertDNSNames"] = etcdPeerCertDNSNames
-	funcs["etcdServerCertCommand"] = etcdServerCertCommand
-	funcs["etcdPeerCertCommand"] = etcdPeerCertCommand
-	funcs["etcdMetricCertCommand"] = etcdMetricCertCommand
 	funcs["cloudProvider"] = cloudProvider
 	funcs["cloudConfigFlag"] = cloudConfigFlag
 	tmpl, err := template.New(path).Funcs(funcs).Parse(string(b))
@@ -371,122 +307,17 @@ func skipMissing(key string) (interface{}, error) {
 	return fmt.Sprintf("{{.%s}}", key), nil
 }
 
-// Process the {{etcdPeerCertDNSNames}} and {{etcdServerCertDNSNames}}
-func etcdServerCertDNSNames(cfg RenderConfig) (interface{}, error) {
-	var dnsNames = []string{
-		"localhost",
-		"etcd.kube-system.svc",                  // sign for the local etcd service name that cluster-network apiservers use to communicate
-		"etcd.kube-system.svc.cluster.local",    // sign for the local etcd service name that cluster-network apiservers use to communicate
-		"etcd.openshift-etcd.svc",               // sign for the local etcd service name that cluster-network apiservers use to communicate
-		"etcd.openshift-etcd.svc.cluster.local", // sign for the local etcd service name that cluster-network apiservers use to communicate
-		"${ETCD_WILDCARD_DNS_NAME}",
-	}
-	return strings.Join(dnsNames, ","), nil
-}
-
-func etcdPeerCertDNSNames(cfg RenderConfig) (interface{}, error) {
-	if cfg.EtcdDiscoveryDomain == "" {
-		return nil, fmt.Errorf("invalid configuration")
-	}
-
-	var dnsNames = []string{
-		"${ETCD_DNS_NAME}",
-		cfg.EtcdDiscoveryDomain, // https://github.com/etcd-io/etcd/blob/583763261f1c843e07c1bf7fea5fb4cfb684fe87/Documentation/op-guide/clustering.md#dns-discovery
-	}
-	return strings.Join(dnsNames, ","), nil
-}
-
-func etcdServerCertCommand(cfg RenderConfig) (interface{}, error) {
-	commands := []string{}
-	if cfg.Images[ClusterEtcdOperatorImageKey] == "" {
-		serverCertDNS, err := etcdServerCertDNSNames(cfg)
-		if err != nil {
-			return nil, err
-		}
-		commands = append(commands, []string{
-			"kube-client-agent \\",
-			"  request \\",
-			"  --kubeconfig=/etc/kubernetes/kubeconfig \\",
-			"  --orgname=system:etcd-servers \\",
-			"  --assetsdir=/etc/ssl/etcd \\",
-			fmt.Sprintf("  --dnsnames=%s \\", serverCertDNS),
-			"  --commonname=system:etcd-server:${ETCD_DNS_NAME} \\",
-			"  --ipaddrs=${ETCD_IPV4_ADDRESS},${ETCD_LOCALHOST_IP} \\",
-		}...)
-	} else {
-		commands = append(commands, []string{
-			"cluster-etcd-operator \\",
-			"  mount \\",
-			"  --assetsdir=/etc/ssl/etcd \\",
-			"  --commonname=system:etcd-server:${ETCD_DNS_NAME} \\",
-		}...)
-	}
-	return commands, nil
-}
-
-func etcdPeerCertCommand(cfg RenderConfig) (interface{}, error) {
-	commands := []string{}
-	if cfg.Images[ClusterEtcdOperatorImageKey] == "" {
-		peerCertDNS, err := etcdPeerCertDNSNames(cfg)
-		if err != nil {
-			return nil, err
-		}
-		commands = append(commands, []string{
-			"kube-client-agent \\",
-			"  request \\",
-			"  --kubeconfig=/etc/kubernetes/kubeconfig \\",
-			"  --orgname=system:etcd-peers \\",
-			"  --assetsdir=/etc/ssl/etcd \\",
-			fmt.Sprintf("  --dnsnames=%s \\", peerCertDNS),
-			"  --commonname=system:etcd-peer:${ETCD_DNS_NAME} \\",
-			"  --ipaddrs=${ETCD_IPV4_ADDRESS} \\",
-		}...)
-	} else {
-		commands = append(commands, []string{
-			"cluster-etcd-operator \\",
-			"  mount \\",
-			"  --assetsdir=/etc/ssl/etcd \\",
-			"  --commonname=system:etcd-peer:${ETCD_DNS_NAME} \\",
-		}...)
-	}
-	return commands, nil
-}
-
-func etcdMetricCertCommand(cfg RenderConfig) (interface{}, error) {
-	commands := []string{}
-	if cfg.Images[ClusterEtcdOperatorImageKey] == "" {
-		metricCertDNS, err := etcdServerCertDNSNames(cfg)
-		if err != nil {
-			return nil, err
-		}
-		commands = append(commands, []string{
-			"kube-client-agent \\",
-			"  request \\",
-			"  --kubeconfig=/etc/kubernetes/kubeconfig \\",
-			"  --orgname=system:etcd-metrics \\",
-			"  --assetsdir=/etc/ssl/etcd \\",
-			fmt.Sprintf("  --dnsnames=%s \\", metricCertDNS),
-			"  --commonname=system:etcd-metric:${ETCD_DNS_NAME} \\",
-			"  --ipaddrs=${ETCD_IPV4_ADDRESS} \\",
-		}...)
-	} else {
-		commands = append(commands, []string{
-			"cluster-etcd-operator \\",
-			"  mount \\",
-			"  --assetsdir=/etc/ssl/etcd \\",
-			"  --commonname=system:etcd-metric:${ETCD_DNS_NAME} \\",
-		}...)
-	}
-	return commands, nil
-}
-
 func cloudProvider(cfg RenderConfig) (interface{}, error) {
-	switch cfg.Platform {
-	case platformAWS, platformAzure, platformOpenStack, platformVSphere:
-		return cfg.Platform, nil
-	case platformGCP:
-		return "gce", nil
-	default:
+	if cfg.Infra.Status.PlatformStatus != nil {
+		switch cfg.Infra.Status.PlatformStatus.Type {
+		case configv1.AWSPlatformType, configv1.AzurePlatformType, configv1.OpenStackPlatformType, configv1.VSpherePlatformType:
+			return strings.ToLower(string(cfg.Infra.Status.PlatformStatus.Type)), nil
+		case configv1.GCPPlatformType:
+			return "gce", nil
+		default:
+			return "", nil
+		}
+	} else {
 		return "", nil
 	}
 }
@@ -503,9 +334,22 @@ func cloudConfigFlag(cfg RenderConfig) interface{} {
 	if cfg.CloudProviderConfig == "" {
 		return ""
 	}
+
+	if cfg.Infra == nil {
+		cfg.Infra = &configv1.Infrastructure{
+			Status: configv1.InfrastructureStatus{},
+		}
+	}
+
+	if cfg.Infra.Status.PlatformStatus == nil {
+		cfg.Infra.Status.PlatformStatus = &configv1.PlatformStatus{
+			Type: "",
+		}
+	}
+
 	flag := "--cloud-config=/etc/kubernetes/cloud.conf"
-	switch cfg.Platform {
-	case platformAWS, platformAzure, platformGCP, platformOpenStack, platformVSphere:
+	switch cfg.Infra.Status.PlatformStatus.Type {
+	case configv1.AWSPlatformType, configv1.AzurePlatformType, configv1.GCPPlatformType, configv1.OpenStackPlatformType, configv1.VSpherePlatformType:
 		return flag
 	default:
 		return ""
